// "Duckey" code
// By Patrick Yukman
// Based on earlier coding by Jon Guinther
// and code from www.cooking-hacks.com

#include <Wire.h>
#include <SoftwareSerial.h>
#include "LSM303DLH.h"

#define GPSrx 9 //rx pin in gps connection
#define GPStx 8 //tx pin in gps connection

//-------------------------------------------------------

SoftwareSerial gps = SoftwareSerial(GPSrx, GPStx);
LSM303DLH compass;

//variables
byte byteGPS = 0;
int i = 0;
int h = 0;
char ID = '#';
char inBuffer[300] = "";
char GPS_GGA[100] = "";
long packetNum = 0;

int sens0 = 0;    //sensors
int sens1 = 1;
int sens2 = 2;
int sens3 = 3;

int val0 = 0;    //aux var
int val1 = 0;
int val2 = 0;
int val3 = 0;

void setup() {
  pinMode(GPSrx, INPUT);
  pinMode(GPStx, OUTPUT);
  gps.begin(4800);
  char CR = 0xD;
  gps.print("$PSTMNMEACONFIG,0,4800,1,1"); // Enables only GGA sentences
  //carriage return (not newline) is necessary for communicating with the gps
  gps.print(CR);
  // See http://www.libelium.com/squidbee/upload/7/75/GPS-Firmware.pdf 
  // for more details
  delay(2000);
  Serial.begin(115200); // This is correct baud rate for squidbee
  Wire.begin();
  compass.enable(); //Turns on LSM303DLH
  Wire.beginTransmission(0x3C >> 1);
  Wire.send(0x01);
  // see table 58 in the datasheet for other gain_setting values
  byte gain_setting = 0b11000000; //first 3 digits correspond to table 58
  Wire.send(gain_setting);
  Wire.endTransmission();
  delay(1000);
}

void loop() {
  val0 = analogRead(sens0);
  val1 = analogRead(sens1);
  val2 = analogRead(sens2);
  val3 = analogRead(sens3);
  compass.read(); //reads mag/acc
  //prints data to radio
  Serial.print("A ");
  Serial.print("X: ");
  dataPrint(compass.a.x);
  Serial.print(" Y: ");
  dataPrint(compass.a.y);
  Serial.print(" Z: ");
  dataPrint(compass.a.z);

  Serial.print(" M ");  
  Serial.print("X: ");
  dataPrint(compass.m.x);
  Serial.print(" Y: ");
  dataPrint(compass.m.y);
  Serial.print(" Z: ");
  dataPrint(compass.m.z);
  
  sendAnalogData(val0,val1,val2,val3);
  Serial.print(" ");
  Serial.print(packetNum);
  packetNum++;
  Serial.print(" ");
  Serial.println(ID);
  GPSloop();
}

void sendAnalogData(int data0,int data1,int data2, int data3) {
  Serial.print(" D0: ");
  dataPrint(data0);
  Serial.print(" D1: ");
  dataPrint(data1);
  Serial.print(" D2: ");
  dataPrint(data2);
  Serial.print(" D3: ");
  dataPrint(data3);
}

void GPSloop() {
  for (int k = 0; k < 100; k++) {
    GPS_GGA[k] = NULL;
  }
  byteGPS = 0;
  byteGPS = gps.read();
  while(byteGPS != 'A'){
    byteGPS = gps.read();
  }
  GPS_GGA[0]='$';
  GPS_GGA[1]='G';
  GPS_GGA[2]='P';    
  GPS_GGA[3]='G';
  GPS_GGA[4]='G';
  GPS_GGA[5]='A';
  
    i = 6;
    while((byteGPS != 42)){      
      byteGPS = gps.read();         
      inBuffer[i]=byteGPS;
      GPS_GGA[i]=byteGPS;
      i++;                      
  }
    
  // print the GGA sentence to USB
  if ((GPS_GGA[62] != NULL) && (GPS_GGA[7] != 'A') && (GPS_GGA[10] != NULL)) {
    h=0;
    while(GPS_GGA[h] != 42){
      Serial.print(GPS_GGA[h]);
      h++;
    }
    Serial.print(" ");
    Serial.println(ID);
  }
}

//Prints each data value, appending zeroes to the front if the value has less than 4 digits. 
//This function only serves the purpose of standardizing the number format of each analog data value.
void dataPrint(int value) {
  int j = 0;
  if (value < 0) {
    value = value*(-1);
    Serial.print("-");
  }
  else {
    Serial.print("+");
  }
  while (((intlen(value)) + j) < 4) {
    Serial.print(0);
    j++;
  }
  Serial.print(value);
}

//Helper function to return the digit length of an int.
int intlen(int start) {
  if (start == 0) {
    return 1;
  }
  else {
    int end = 0; 
    while(start > 0) { 
        start = start/10; 
        end++; 
    } 
    return end; 
  }
}
